/*
 * cfg.h
 *
 *  Created on: 7. 10. 2017
 *      Author: Milan
 */
#include "chip.h"
#include <stdbool.h>

#ifndef CFG_H_
#define CFG_H_


#define TARGET_LPC11UXX


#define WDT_RESET()			Chip_WWDT_Feed(LPC_WWDT)
#define US_TICK				12 // (1us / ( 1 / oscilator_clock))
#define LPC_SSP          	LPC_SSP0

#define VDD					3012.0
#define ADC_RESOLUTION		1023.0 // 2^10b -1

#define EXT_PWR_DIVIDER		(220.0 + 680.0) / (220.0)
#define USB_DIVIDER			(220.0 + 220.0) / (220.0)
#define RF_DIVIDER			(100.0 + 100.0) / (100.0)
#define BATT_DIVIDER		(680.0 + 390.0) / (390.0)

#define ADC_NUM_OF_CH		5
#define ADC_TAU				10
#define EXT_PWR_TRESHOLD	10000
#define BATT_CHARGE_TRESHOLD 6500
#define BATTERY_CHARGED		7200
#define STAT_DET_TRESHOLD	2000
#define USB_VOLTAGE_TRESHOLD 4000

#define SYSTICK_RATE 		1000 //  interupt period = 1 / SYSTICK_RATE

/* Transmit and receive ring buffer sizes */
#define UART_SRB_SIZE 128	/* Send */
#define UART_RRB_SIZE 32	/* Receive */
#define CMD_RX_LENGTH 5
#define BUFF_LENGTH	9

/* Attenuator  */
#define ATTENUATION_MAX	31
#define ATT_ADDRESS		0x00

/*EEPROM ACCESS*/
#define TARGET_EEPROM_ADDRESS   64
#define MEM_SIZE        32
#define ATT_INTEGERS	0
#define ATT_TENTHS	1

typedef enum adc_ch{
	adc_ext_pwr = ADC_CH0,
	adc_stat_det = ADC_CH1,
	adc_usb_voltage = ADC_CH3,
	adc_rf_voltage = ADC_CH5,
	adc_battery_voltage = ADC_CH6,
}ADC_ch;

typedef struct {
	uint32_t pure, sum1st, sum2nd, filtered;
} ADC_val;

typedef struct {
	uint16_t ext_pwr, stat_det, usb_voltage, rf_voltage, battery_voltage;
} Voltage;

typedef struct {
	uint8_t ms:1,
			sec:1,
			charging:1,
			batt_charged:1,
			batt_need_charging:1,
			charging_err:1,
			uart:1,
			attenuation_set:1;
} Flags;

typedef struct{
	uint16_t iap_return_code;
	uint16_t actual_attenuation;
} EEPROM;

typedef enum state {
  charging,
  charging_err,
  batt_need_charging,
  normal_operation,
  pc_connected,
} State;

typedef enum cmd {
	attenuation_set = 0x01,
} Cmd;

extern State state;
extern ADC_ch adc_ch;
extern Flags flags;
extern ADC_val adc_val[7];
extern Voltage voltage;
extern RINGBUFF_T txring, rxring;
extern EEPROM eeprom;

#endif /* CFG_H_ */
