/*
 * adc.c
 *
 *  Created on: 10. 10. 2017
 *      Author: Milan
 */

#include "adc.h"

static ADC_ch adc_ch_arr[] = {adc_ext_pwr,adc_stat_det,adc_usb_voltage,adc_rf_voltage,adc_battery_voltage};

static void ADC_conversion(ADC_ch channel)
{
	switch(channel)
	{
		case adc_ext_pwr:
			voltage.ext_pwr = ((double)adc_val[channel].filtered / ADC_RESOLUTION) * VDD * EXT_PWR_DIVIDER;
			break;
		case adc_stat_det:
			voltage.stat_det = ((double)adc_val[channel].filtered / ADC_RESOLUTION) * VDD;
			break;
		case adc_usb_voltage:
			voltage.usb_voltage = ((double)adc_val[channel].filtered / ADC_RESOLUTION) * VDD * USB_DIVIDER;
			break;
		case adc_rf_voltage:
			voltage.rf_voltage = ((double)adc_val[channel].filtered / ADC_RESOLUTION) * VDD * RF_DIVIDER;
			break;
		case adc_battery_voltage:
			voltage.battery_voltage = ((double)adc_val[channel].filtered / ADC_RESOLUTION) * VDD * BATT_DIVIDER;
			break;
	}
}

static void ADC_filtering(ADC_ch channel)
{
	adc_val[channel].sum1st += adc_val[channel].pure - (adc_val[channel].sum1st / ADC_TAU);
	adc_val[channel].sum2nd += adc_val[channel].sum1st - (adc_val[channel].sum2nd / ADC_TAU);
	adc_val[channel].filtered = adc_val[channel].sum2nd / (ADC_TAU * ADC_TAU);
}

/* Routine to read used channel */
static uint8_t ADC_Read_Channel(ADC_ch channel)
{
	Chip_ADC_EnableChannel(LPC_ADC, channel, ENABLE);
	Chip_ADC_SetStartMode(LPC_ADC, ADC_START_NOW, ADC_TRIGGERMODE_RISING);

	if (Chip_ADC_ReadStatus(LPC_ADC, channel, ADC_DR_DONE_STAT) == SET)
	{

		Chip_ADC_ReadValue(LPC_ADC,channel,&adc_val[channel].pure);
		ADC_filtering(channel);
		ADC_conversion(channel);
		Chip_ADC_EnableChannel(LPC_ADC, channel, DISABLE);
		return 1;
	}
	return 0;
}

extern void ADC_read_channel_fast(ADC_ch channel)
{
	Chip_ADC_EnableChannel(LPC_ADC, channel, ENABLE);
	Chip_ADC_SetStartMode(LPC_ADC, ADC_START_NOW, ADC_TRIGGERMODE_RISING);
	while (Chip_ADC_ReadStatus(LPC_ADC, channel, ADC_DR_DONE_STAT) != SET) {}
	Chip_ADC_ReadValue(LPC_ADC, channel, &adc_val[channel].filtered);
	ADC_conversion(channel);
	Chip_ADC_EnableChannel(LPC_ADC, channel, DISABLE);
}
extern void ADC_read_all_channels(void)
{
	static uint8_t adc_ch_arr_p;
	if(ADC_Read_Channel(adc_ch_arr[adc_ch_arr_p]))
	{
		if(adc_ch_arr_p != (ADC_NUM_OF_CH-1))
			adc_ch_arr_p++;
		else
			adc_ch_arr_p = 0;
	}
}


