/*
 * handlers.c
 *
 *  Created on: 10. 10. 2017
 *      Author: Milan
 */

#include "handlers.h"

extern void Signalization_handler(void)
{
	switch(state)
	{
		case charging:
			LED_GREEN_BLINKING;
			LED_RED(false);
			break;
		case batt_need_charging:
			LED_RED_BLINKING;
			LED_GREEN(false);
			break;
		case charging_err:
			LED_GREEN_BLINKING;
			LED_RED_BLINKING;
			break;
		case normal_operation:
			LED_GREEN(true);
			LED_RED(false);
			break;
		case pc_connected:
			if(flags.charging)
			{
				LED_GREEN_BLINKING;
				LED_RED(true);
			}
			else
			{
				LED_GREEN(true);
				LED_RED(true);
			}
			break;
		default:
			LED_GREEN(false);
			LED_RED(false);
			break;
	}
}

extern void Charge_handler(void)
{
	if((voltage.ext_pwr < EXT_PWR_TRESHOLD) && (voltage.battery_voltage > BATTERY_CHARGED))
	{
		flags.charging = false;
		flags.batt_charged = true;
		flags.batt_need_charging = false;
		if(state != pc_connected)
		state = normal_operation;
		RF_SUPPLY_EN(true);
	}
	else if((voltage.ext_pwr < EXT_PWR_TRESHOLD) && (voltage.battery_voltage < BATTERY_CHARGED))
	{
		flags.charging = false;
		flags.batt_need_charging = true;
		flags.batt_charged = false;
		state = batt_need_charging;
		RF_SUPPLY_EN(true);
	}

	if(voltage.ext_pwr > EXT_PWR_TRESHOLD)
	{

		if(!flags.charging)
		{
			CHARGE_CURRENT_SET(true);
			delay_us(750);  // nahradit.. je nutno počkat prostě 1 kolo a zeptat se znovu
			ADC_read_channel_fast(adc_stat_det);

			if(voltage.stat_det > STAT_DET_TRESHOLD)
			{
				CHARGE_CURRENT_SET(false);
				flags.charging = false;
			}
			else
			{
				flags.charging = true;
				flags.batt_charged = false;
				state = charging;
				RF_SUPPLY_EN(false);
			}
		}
		else if((voltage.stat_det > STAT_DET_TRESHOLD) && (flags.charging == true))
		{
			flags.charging = false;
			flags.batt_need_charging = false;
			flags.batt_charged = true;
			state = normal_operation;
			CHARGE_CURRENT_SET(false);
			RF_SUPPLY_EN(true);
		}
		else
		{
			// charging succesfully running
		}
	}
	else if((voltage.ext_pwr > EXT_PWR_TRESHOLD) && (voltage.battery_voltage < BATT_CHARGE_TRESHOLD))
	{
		state = charging_err;  // napětí na vstupu je, ale na výstupu IO ne-e
	}
}

extern void PC_detection_handler(void)
{
	if(voltage.usb_voltage > USB_VOLTAGE_TRESHOLD && (!flags.uart))
	{
		flags.uart = true;
		Chip_UART_Init(LPC_USART);
		RF_SUPPLY_EN(false);
		state = pc_connected;
	}
	else if((voltage.usb_voltage < USB_VOLTAGE_TRESHOLD) && ((state == pc_connected) || (state == batt_need_charging)))
	{
		flags.uart = false;
		Chip_UART_DeInit(LPC_USART);

		if(flags.charging)
		{
			state = charging;
			RF_SUPPLY_EN(false);
		}
		else if(flags.batt_need_charging)
		{
			RF_SUPPLY_EN(true);
			state = batt_need_charging;
		}
		else if(flags.charging_err)
		{
			RF_SUPPLY_EN(true);
			state = charging_err;
		}
		else
		{
			state = normal_operation;
			RF_SUPPLY_EN(true);
		}
	}
}

void Uart_cmd_processing(uint8_t *p_rx_buff)
{
	uint8_t cmd_rq = ((p_rx_buff[0]) - 48);
	uint16_t cmd_data = ( (1000 * (p_rx_buff[1] - 48)) + (100 * (p_rx_buff[2] - 48)) + (10 * (p_rx_buff[3] - 48)) + (1 * (p_rx_buff[4] - 48)));


	switch(cmd_rq)
	{
	case attenuation_set:
		flags.attenuation_set = false;

		for(uint8_t i=0; i < CMD_RX_LENGTH;i++)
			p_rx_buff[i] = 0;
		sky12343_driver((cmd_data / 100),(cmd_data % 100));
		//saving data into the eeprom memory -> 'll be readed after reset and set as default
		mem[ATT_INTEGERS] = cmd_data / 100;
		mem[ATT_TENTHS] = 25*((cmd_data % 100) / 25);
		eeprom.iap_return_code = IAP_write_eeprom(mem, (char*)TARGET_EEPROM_ADDRESS, MEM_SIZE );
		if(eeprom.iap_return_code == 0)
			eeprom.actual_attenuation = (100 * (cmd_data / 100)) + (25 *((cmd_data % 100) / 25));
		break;
	default:
		break;
	}

}

extern void Uart_handler(void)
{
	static uint16_t start_byte[] = {0xAA11};
	static uint16_t buff[BUFF_LENGTH];
	static uint8_t rx_buff[CMD_RX_LENGTH];

	if(flags.uart)
	{
		uint16_t *p_buff = buff;
		memcpy(p_buff,start_byte,sizeof(start_byte));
		p_buff += (sizeof(start_byte) / sizeof(start_byte[0]));
		memcpy(p_buff, &voltage, sizeof(voltage));
		p_buff += sizeof(voltage)/sizeof(uint16_t);
		memcpy(p_buff,&eeprom,sizeof(eeprom));
		p_buff += sizeof(eeprom)/sizeof(uint16_t);
		memcpy(p_buff,&flags,sizeof(flags));
		Chip_UART_SendRB(LPC_USART,&txring.data,&buff,sizeof(buff));
		Chip_UART_ReadRB(LPC_USART, &rxring.data, &rx_buff, sizeof(rx_buff));
		Uart_cmd_processing(rx_buff);
	}
}



