 #include <string.h>
 #include <stdbool.h>
 #include "chip.h"
 #include "pins.h"

/* IOCON pin definitions for pin muxing */
typedef struct {
	uint32_t port : 8;			/* Pin port */
	uint32_t pin : 8;			/* Pin number */
	uint32_t modefunc : 16;		/* Function and mode */
} PINMUX_GRP_T;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/* Pin muxing table, only items that need changing from their default pin
   state are in this table.
*/
STATIC const PINMUX_GRP_T pinmuxing[] = {
	{0,6, (IOCON_FUNC2 | IOCON_MODE_INACT)},	/* PIO0_6 used for SCK0 */
	{0,9, (IOCON_FUNC1 | IOCON_MODE_INACT)},	/* PIO0_9 used for MOSI */
	{0,11, (IOCON_FUNC2 | IOCON_MODE_INACT)},	/* PIO1_0 used for AD0 */
	{0,12, (IOCON_FUNC2 | IOCON_ADMODE_EN)},	/* PIO1_0 used for AD1 */
	{0,14, (IOCON_FUNC2 | IOCON_ADMODE_EN)},	/* PIO1_2 used for AD3 */
	{0,16, (IOCON_FUNC1 | IOCON_ADMODE_EN)},	/* PIO1_4 used for AD5 */
	{0,22, (IOCON_FUNC1 | IOCON_ADMODE_EN)},	/* PIO1_4 used for AD6 */
	{0,18,  (IOCON_FUNC1 | IOCON_MODE_INACT)},	/* PIO1_6 used for RXD */
	{0,19,  (IOCON_FUNC1 | IOCON_MODE_INACT)},	/* PIO1_7 used for TXD */
	{0,13,  (IOCON_FUNC1 | IOCON_MODE_INACT)},	/* PIO0_13 used for GPIO */
};

extern void IO_Init(void)
{
	Chip_GPIO_SetPinDIROutput(LPC_GPIO, 0, 13);	/* RF supply en */
	Chip_GPIO_SetPinDIROutput(LPC_GPIO, 0, 7);	/* Charge Current set  */
	Chip_GPIO_SetPinDIROutput(LPC_GPIO, 0, 2);	/* LATCH */
	Chip_GPIO_SetPinDIROutput(LPC_GPIO, 1, 19);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO, 0, 17);

	CHARGE_CURRENT_SET(false);
	RF_SUPPLY_EN(true);
	ATT_LATCH(false);
}

extern void SystemSetupMuxing(void)
{
	static int i;
	/* Enable IOCON clock */
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	IO_Init();
	for (i = 0; i < (sizeof(pinmuxing) / sizeof(PINMUX_GRP_T)); i++) {
			Chip_IOCON_PinMuxSet(LPC_IOCON, pinmuxing[i].port, pinmuxing[i].pin,
								 pinmuxing[i].modefunc);
		}

}
