/*
 * @brief Common SystemInit function for LPC11xx chips
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012-14
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

 //#include "chip.h"
 #include "pins.h"
 #include "cfg.h"
 #include "sysinit.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/
const uint32_t OscRateIn = 12000000;
const uint32_t ExtRateIn = 12000000;


static ADC_CLOCK_SETUP_T 	ADCSetup;
static uint8_t rxbuff[UART_RRB_SIZE], txbuff[UART_SRB_SIZE];


/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

extern void delay_us(uint16_t us)
{
	for(uint16_t i = 0; i < (12 * us); i++)
	{
		asm("nop");
	}

}

/*	Enable Using External Crystal	*/
static void SystemClockInit(void)
{
	Chip_Clock_SetPLLBypass(1, 1);

	Chip_Clock_SetMainClockSource(SYSCTL_MAINCLKSRC_PLLIN);
	Chip_Clock_SetSysClockDiv(1);

}
/* Set the Watchdog timer approximately to 2s; No Interupt*/
static void WDT_Setting(void)
{
	if(!DEBUG_MODE)
	{
		static uint32_t wdtFreq;
		Chip_WWDT_Init(LPC_WWDT);
		Chip_SYSCTL_PowerUp(SYSCTL_POWERDOWN_WDTOSC_PD);
		Chip_Clock_SetWDTOSC(WDTLFO_OSC_1_05, 20);
		wdtFreq = Chip_Clock_GetWDTOSCRate() / 4;
		Chip_WWDT_SelClockSource(LPC_WWDT, WWDT_CLKSRC_WATCHDOG_WDOSC);
		Chip_WWDT_SetTimeOut(LPC_WWDT, wdtFreq * 2);
		Chip_WWDT_SetOption(LPC_WWDT, WWDT_WDMOD_WDRESET); //reset when timeout
		Chip_WWDT_Feed(LPC_WWDT); // Start WDT
	}
}

static void UART_Setting(void)
{

	Chip_UART_Init(LPC_USART);
	Chip_UART_SetBaud(LPC_USART, 9600);
	Chip_UART_ConfigData(LPC_USART, (UART_LCR_WLEN8 | UART_LCR_SBS_1BIT)); // 8N1 frame
	Chip_UART_SetupFIFOS(LPC_USART, (UART_FCR_FIFO_EN | UART_FCR_TRG_LEV2));
	Chip_UART_TXEnable(LPC_USART);

	/* Before using the ring buffers, initialize them using the ring
 	buffer init function */
	RingBuffer_Init(&rxring, rxbuff, 1, UART_RRB_SIZE);
	RingBuffer_Init(&txring, txbuff, 1, UART_SRB_SIZE);
	Chip_UART_IntEnable(LPC_USART, (UART_IER_RBRINT | UART_IER_RLSINT));

	NVIC_SetPriority(UART0_IRQn, 1);
	NVIC_EnableIRQ(UART0_IRQn);

	Chip_UART_DeInit(LPC_USART);

}

static void SPI_Setting(void)
{
	Chip_SSP_Init(LPC_SSP);
	Chip_SSP_Set_Mode(LPC_SSP, SSP_MODE_MASTER);
	Chip_SSP_SetFormat(LPC_SSP, SSP_BITS_16, SSP_FRAMEFORMAT_SPI, SSP_CLOCK_CPHA0_CPOL0);
	Chip_SSP_SetBitRate(LPC_SSP, 100000);
	Chip_SSP_Enable(LPC_SSP);
}


extern void MySystemInit(void)
{

	SystemClockInit();
	SystemCoreClockUpdate();
	SystemSetupMuxing();

	Chip_GPIO_Init(LPC_GPIO);
	SPI_Setting();
	Chip_ADC_Init(LPC_ADC, &ADCSetup);

	UART_Setting();
	SysTick_Config(SystemCoreClock / SYSTICK_RATE);

	WDT_Setting();

	flags.batt_need_charging = false;
	flags.charging = false;
	flags.charging_err = false;
	flags.uart = false;
	flags.attenuation_set = false;
	state = normal_operation;
}

/* Set up and initialize hardware prior to call to main */
extern void SystemInit(void)
{
#if defined(NO_BOARD_LIB)
	/* Chip specific SystemInit */
	Chip_SystemInit();
#else
	/* Board specific SystemInit */
	Board_SystemInit();
#endif
}
